local _,stash = ...
local L = stash.localization
local C = stash.constants
local settings = stash.settings
_stash = stash -- for global reference and debugging

stash.main = CreateFrame("Frame")
stash.main:RegisterEvent("PLAYER_LOGIN")
stash.main:SetScript("OnEvent",function(self,event,...)
    if self[event] then
        self[event](self,...)
    end
end)

-- login initialization
function stash.main:PLAYER_LOGIN()
    stash.saved = StashSaved -- savedvar for all items across all characters
    self:CreateCharacter()
    -- at one point, filters were going to be remembered across sessions; keeping them a savedvar in case that happens
    settings.FilterRealm = false
    settings.FilterCharacter = false
    settings.FilterLocation = false
    settings.FilterType = false
    settings.FilterSubType = false
    settings.FilterEquipSlot = false
    -- finish setting up panels and stuff on login
    stash.frame:Setup()
    stash.itemsPanel:Setup()
    stash.filterPanel:Setup()
    stash.moneyPanel:Setup()
    stash.settingsPanel:Setup()
    stash.confirmPanel:Setup()
    stash.minimapButton:Setup()
    stash.tracking:Setup()
    stash.tooltips:Setup()

    local ldb = LibStub and LibStub.GetLibrary and LibStub:GetLibrary("LibDataBroker-1.1",true)
    if ldb then
        ldb:NewDataObject("Stash",{type="launcher", icon="Interface\\Icons\\INV_Misc_Bag_07", tooltiptext=L["Stash"], OnClick=stash.frame.Toggle }) 
    end

    SLASH_STASH1 = "/stash"
    SlashCmdList["STASH"] = stash.main.SlashCmd
end

function stash.main:MODIFIER_STATE_CHANGED()
    if stash.itemsPanel:IsVisible() and MouseIsOver(stash.itemsPanel.List) then
        for _,button in ipairs(stash.itemsPanel.List.ScrollFrame.Buttons) do
            if GetMouseFocus()==button then
                if IsModifiedClick("DRESSUP") and stash.itemsPanel:IsDressUpable(button.itemID) then
                    ShowInspectCursor()
                else
                    ResetCursor()
                end
                return
            end
        end
    end
end

-- this creates a character in StashSaved--if needed--for the currently-logged in player to track their stuff
-- and also sets stash.current to point to the stash of the currently-logged-in player (mostly for tracking updates)
function stash.main:CreateCharacter()
    local realm = GetRealmName()
    local player = GetUnitName("player")
    if not stash.saved[realm] then
        stash.saved[realm] = {}
    end
    if not stash.saved[realm][player] then
        stash.saved[realm][player] = {
            realm = realm,
            player = player,
            money = 0,
            worn = {},
            bags = CopyTable(C.INVENTORY_BAGS), -- becomes [0]={}, [1]={}, etc.
            bank = CopyTable(C.BANK_BAGS), -- becomes [-1]={}, [5]={}, etc.
            reagents = {[-3]={}},
            void = {{},{}},
            mail = {},
            currency = {}
        }
    end
    stash.current = stash.saved[realm][player] -- all item tracking is done on current character's stash only
    stash.current.update = date("%x") -- log current date as last day character was updated
    stash.current.faction = UnitFactionGroup("player") -- log faction/race/gender that could potentially change
    stash.current.race = select(3,UnitRace("player"))
    stash.current.gender = UnitSex("player")
    -- fix tables for current character if they got messed up/added
    -- (tables can be a bit messed up for other characters; but should be right structure for current player
    -- so tracking can put stuff where they belong.)
    stash.main:ValidateCharacter(stash.current,realm,player)
end

-- receives a stash.saved[realm][player] table and validates/adjusts/updates its structure
-- note: this shouldn't be called on guild banks which are fine with many tables missing
function stash.main:ValidateCharacter(character,realm,player)
    assert(type(character)=="table","Invalid saved data for "..(player or "nil").."-"..(realm or "nil"))
    -- validate worn
    if type(character.worn)~="table" then
        character.worn = {}
    end
    -- validate inventory bags
    if type(character.bags)~="table" then
        character.bags = CopyTable(C.INVENTORY_BAGS)
    else -- bags is a table, now confirm it has all the right sub-tables
        for bag in pairs(C.INVENTORY_BAGS) do
            if not character.bags[bag] then
                character.bags[bag] = {} -- bag is missing, create one
            end
        end
        for bag in pairs(character.bags) do
            if not C.INVENTORY_BAGS[bag] then
                character.bags[bag] = nil -- an extra bag somehow, remove it
            end
        end
    end
    -- validate bank bags
    if type(character.bank)~="table" then
        character.bank = CopyTable(C.BANK_BAGS)
    else -- bank is a table, now confirm it has all the right sub-tables
        -- check if this is an older character save being loaded into a 10.x+ client
        if C.IS_MODERN_CLIENT and character.bank[5] then
            for i=12,6,-1 do -- shift bank bags 5->11 to 6->12
                if type(character.bank[i-1])=="table" then
                    character.bank[i] = CopyTable(character.bank[i-1])
                else
                    character.bank[i] = {}
                end
            end
            character.bank[5] = nil -- from 9.2.7 to 10.0.0, bag 5 is moving to inventory bags
        end
        -- now confirm subtables
        for bag in pairs(C.BANK_BAGS) do
            if not character.bank[bag] then
                character.bank[bag] = {} -- bag is missing, create one
            end
        end
        for bag in pairs(character.bank) do
            if not C.BANK_BAGS[bag] then
                character.bank[bag] = nil -- an extra bag somehow, remove it
            end
        end
    end
    -- validate reagents
    if type(character.reagents)~="table" or type(character.reagents[-3])~="table" then
        character.reagents = {[-3]={}}
    end
    -- validate void bank
    if type(character.void)~="table" or type(character.void[1])~="table" or type(character.void[2])~="table" then
        character.void = {{},{}}
    end
    -- validate mail
    if type(character.mail)~="table" then
        character.mail = {}
    end
    -- validate currency
    if type(character.currency)~="table" then
        character.currency = {}
    end
end

-- /stash command toggles but will also search for a term if one given on command line
function stash.main.SlashCmd(msg)
    -- if an argument is given, doing a search and always show the window (if not already shown)
    if msg and msg:trim():len()>0 then
        local searchBox = stash.itemsPanel.Top.SearchBox
        stash.frame:Show()
        searchBox:SetText(msg:trim())
        searchBox:GetScript("OnEditFocusLost")(searchBox) -- handle clearbutton/instructions
    else -- if no argument, toggle the window
        stash.frame:Toggle()
    end
end

-- bindings.xml constants
BINDING_HEADER_STASH = L["Stash"]
BINDING_NAME_STASH_TOGGLE = L["Show/Hide Stash"]

